/**
  ******************************************************************************
  * @file    IO_Toggle/main.c 
  * @author  MCD Application Team
  * @version V1.0.0Delay(100000);
  * @date    19-September-2011
  * @brief   Main program body
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************  
  */ 

/* Includes ------------------------------------------------------------------*/
#include "stm32f10x.h"
#include <stdio.h>
#include <stdlib.h>
#include "picojpeg.h"
#include "imagedata.c"
#include "main.h"
/** @addtogroup STM32F4_Discovery_Peripheral_Examples
  * @{
  */

/** @addtogroup IO_Toggle
  * @{
  */

#ifndef max
#define max(a,b)    (((a) > (b)) ? (a) : (b))
#endif
#ifndef min
#define min(a,b)    (((a) < (b)) ? (a) : (b))
#endif

//#define USE_DBPRINTF
#ifdef USE_DBPRINTF
#define db_puts(s)                   puts(s)
#define db_printf(szFormat, ...)     printf(szFormat,##__VA_ARGS__)
#else
#define db_out(s)
#define db_printf(szFormat, ...)
#endif


/* Private typedef -----------------------------------------------------------*/
GPIO_InitTypeDef  GPIO_InitStructure;

/* Private define ------------------------------------------------------------*/
#define LOCAL_IMAGE_BUFFER_SIZE 4096
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
uint32_t localImageBufferCursor = 0;
uint8_t LEDBlinkStatus = 0;
unsigned int arrayCursor = 0;

#define BUFFER_INDEX_START 0
//keep track of which buffer is currently being read from
static uint8_t readBufferIndex = BUFFER_INDEX_START;
static uint8_t writeBufferIndex = BUFFER_INDEX_START;
static uint8_t done = 0;
//main loop already filled the first part of &picture
static unsigned char * localBufferAddr = &picture[BUFFER_INDEX_START][0];
static unsigned char * altBufferAddr = &picture[BUFFER_INDEX_START][0];

/* Private function prototypes -----------------------------------------------*/
void Delay(__IO uint32_t nCount);
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Main program
  * @param  None
  * @retval None
  */

static uint32_t g_nInFileSize;
static uint32_t g_nInFileOfs;



unsigned char * readBuffer(uint32_t start, uint32_t bytes, unsigned char * buffAddr) {
    uint32_t i;
    uint8_t skipFour=0;
    
    //this camera is fucked
    //if the starting address is not divisible by 8
    //it starts transmitting from 4 bytes before target
    //Ignore those bytes.
    if ((start%8)!=0) {
        skipFour = 1;
        bytes+=4;
    }
    
    
    //TODO: Figure out why data register is not empty before the second image is taken
    //maybe it just picks up jumk.  Prolly should clear it out before continuing.
    
    //wait for DR to finish up before starting DMA.
    while (USART_GetFlagStatus(USART2,USART_FLAG_RXNE)) {
        USART_ReceiveData(USART2);
    }
    
    uint8_t ignoreBytes = 0;
    //ignore signal response bytes
    ignoreBytes +=5;
    
    //ignore four bytes if you need to
    if (skipFour)
    {
        ignoreBytes+=4;
        bytes-=4;
    }
    
    
    /*Configure camera DMA*/
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
    USART_DMACmd(USART2,USART_DMAReq_Rx,ENABLE);
    DMA_InitTypeDef DMA_InitStructure;
    DMA_DeInit(DMA1_Channel6);
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
    DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)buffAddr;
    DMA_InitStructure.DMA_BufferSize =(uint16_t)bytes+ignoreBytes+5; //the last five bytes are the tail.
    DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)&USART2->DR;
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
    DMA_InitStructure.DMA_Priority = DMA_Priority_High;
    DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
    DMA_Init(DMA1_Channel6,&DMA_InitStructure);
    DMA_Cmd(DMA1_Channel6, ENABLE);
    
    
    
    sendcam(0x56);
    sendcam(0x0);
    sendcam(0x32);
    sendcam(0x0C);
    sendcam(0x00);
    sendcam(0x0A);
    sendcam((start>>24)&0xff);
    sendcam((start>>16)&0xff);
    sendcam((start>>8 )&0xff);
    sendcam((start    )&0xff);
    sendcam((bytes>>24)&0xff);
    sendcam((bytes>>16)&0xff);
    sendcam((bytes>>8)&0xff);
    sendcam((bytes)&0xff);//32 bytes at a time
    sendcam(0);
    sendcam(0);//small delay (about .3ms)
    
    
    //for (i=0;i<(bytes);i++){
    //    picture[i]=reccam();
    //}
    return &buffAddr[ignoreBytes];
    
    
}


unsigned char copyArray(unsigned char* pBuf,unsigned int *start, unsigned char count, const unsigned char* sourceArray) {
    int i;
    for (i=0;i<count;i++) {
        pBuf[i]=sourceArray[i+*start];
    }
    *start+=count;
    return count;
}

uint8_t DMABusy() {
    if (DMA_GetFlagStatus(DMA1_FLAG_TC6)==0)
        return (uint8_t)1;
    else
        return (uint8_t)0;
}

#define PJPG_MAX_IN_BUF_SIZE 256

unsigned char pjpeg_need_bytes_callback(unsigned char* pBuf, unsigned char buf_size, unsigned char *pBytes_actually_read, void *pCallback_data)
{

    
    uint32_t n;

    //if DMA is free, start it up again
    if (!DMABusy()&&(readBufferIndex ==writeBufferIndex)&&(done==0)) {
        if (writeBufferIndex ==1)
            writeBufferIndex = 0;
        else
            writeBufferIndex = 1;
        
        //grab the next chunk of the image including some extra bytes
        //at the beginning to allow for a graceful handoff.
        if (localImageBufferCursor+(2*LOCAL_IMAGE_BUFFER_SIZE)<(g_nInFileSize)) {
            altBufferAddr = readBuffer(localImageBufferCursor-(PJPG_MAX_IN_BUF_SIZE)+LOCAL_IMAGE_BUFFER_SIZE,LOCAL_IMAGE_BUFFER_SIZE+(PJPG_MAX_IN_BUF_SIZE), &picture[writeBufferIndex][0]);
        }
        else {
            altBufferAddr = readBuffer(localImageBufferCursor+LOCAL_IMAGE_BUFFER_SIZE-(PJPG_MAX_IN_BUF_SIZE),(g_nInFileSize-(localImageBufferCursor+LOCAL_IMAGE_BUFFER_SIZE))+PJPG_MAX_IN_BUF_SIZE,&picture[writeBufferIndex][0]);
            done = 1;
        }
    }
    
    //n is the largest amount of data that the JPEG decoder
    //is going to take this round
    n = min(g_nInFileSize - g_nInFileOfs, buf_size);


    //See if you have enough data to work with locally.  If you don't
    //switch to the other buffer
    if ((n+g_nInFileOfs)>LOCAL_IMAGE_BUFFER_SIZE+localImageBufferCursor) {
        
        while(DMABusy())
        {}
        
        
        if (readBufferIndex ==1)
            readBufferIndex = 0;
        else
            readBufferIndex = 1;

        
        //buffers all have a PJPG_MAX_IN_BUF_SIZE leader on them to ease transition.
        //need to step back from that whatever amount.
        localImageBufferCursor += LOCAL_IMAGE_BUFFER_SIZE;
        arrayCursor= ((g_nInFileOfs)-localImageBufferCursor+(PJPG_MAX_IN_BUF_SIZE));
        localBufferAddr = altBufferAddr;
        
    }
    
    //Delay(1000);
    
    
    if (n && (copyArray(pBuf, &arrayCursor, n, localBufferAddr) != n))
    {
        return PJPG_STREAM_READ_ERROR;
    }
    *pBytes_actually_read = (unsigned char)(n);
    g_nInFileOfs += n;
    return 0;
}

void donothing()
{
    int i;
    i++;
}

typedef enum {
    ON,
    OFF,
    BLINK,
    DONTBLINK
} ledStatus_t;

typedef enum {
    RED = GPIO_Pin_9,
    GREEN = GPIO_Pin_10
} ledColors_t;



void setLED(ledColors_t color, ledStatus_t status) {
    if (status == ON) {
        GPIO_SetBits(GPIOA,color);
        if (color ==RED)
            LEDBlinkStatus&=~1;
        else
            LEDBlinkStatus&=~2;
    }
    else if (status == OFF) {
        GPIO_ResetBits(GPIOA,color);
        if (color ==RED)
            LEDBlinkStatus&=~1;
        else
            LEDBlinkStatus&=~2;
    }
    else if (status == BLINK) {
        if (color==RED)
            LEDBlinkStatus |= 1;
        else
            LEDBlinkStatus |= 2;
    }
}

uint8_t getLED(ledColors_t color) {
    return (GPIO_ReadOutputDataBit(GPIOA,color));
}

void send(uint8_t data) {
    //make sure the last byte made it out
    while(!USART_GetFlagStatus(UART4, USART_FLAG_TXE)){};
    USART_SendData(UART4, data);
    
}


void sendcam(uint8_t data) {
    //make sure the last byte made it out
    while(!USART_GetFlagStatus(USART2, USART_FLAG_TXE)){};
    USART_SendData(USART2, data);
    
}

void waitForDMAComplete() {
    while(DMA_GetFlagStatus(DMA1_FLAG_TC6) == 0)
    {
        //DMACount = DMA_GetCurrDataCounter(DMA1_Channel6);
    }
    USART_DMACmd(USART2,USART_DMAReq_Rx,DISABLE);
}
            
void setPrinter(uint8_t status)
{
    if (status)
        GPIO_SetBits(GPIOC, GPIO_Pin_3);
    else
        GPIO_ResetBits(GPIOC,GPIO_Pin_3);
}

void setCamera(uint8_t status)
{
    if (status)
        GPIO_ResetBits(GPIOC, GPIO_Pin_1);
    else
        GPIO_SetBits(GPIOC,GPIO_Pin_1);
}

void initThermalPrinter()
{
    setPrinter(1);
    Delay(0x1FFFFF);
    
    // Printer may have been idle for a very long time, during which the
    // micros() counter has rolled over.  To avoid shenanigans, reset the
    // timeout counter before issuing the wake command.
    //send(255);
    // Datasheet recomments a 50 mS delay before issuing further commands,
    // but in practice this alone isn't sufficient (e.g. text size/style
    // commands may still be misinterpreted on wake).  A slightly longer
    // delay, interspersed with ESC chars (no-ops) seems to help.
    
    
    
    uint8_t i;
    for(i=0; i<10; i++) {
        send(27);
        Delay(10000);
    }

    //reset printer to default state
    send(27);
    send(64);
    
    
    
    // Description of print settings from page 23 of the manual:
    // ESC 7 n1 n2 n3 Setting Control Parameter Command
    // Decimal: 27 55 n1 n2 n3
    // Set "max heating dots", "heating time", "heating interval"
    // n1 = 0-255 Max printing dots, Unit (8dots), Default: 7 (64 dots)
    // n2 = 3-255 Heating time, Unit (10us), Default: 80 (800us)
    // n3 = 0-255 Heating interval, Unit (10us), Default: 2 (20us)
    // The more max heating dots, the more peak current will cost
    // when printing, the faster printing speed. The max heating
    // dots is 8*(n1+1).  The more heating time, the more density,
    // but the slower printing speed.  If heating time is too short,
    // blank page may occur.  The more heating interval, the more
    // clear, but the slower printing speed.

    send(27);
    send(55);   // Esc 7 (print settings)
    
    
    send(20);       // Heating dots (20=balance of darkness vs no jams)
    send(200); // Library default = 255 (max)
    send(250);
    
    // Description of print density from page 23 of the manual:
    // DC2 # n Set printing density
    // Decimal: 18 35 n
    // D4..D0 of n is used to set the printing density.  Density is
    // 50% + 5% * n(D4-D0) printing density.
    // D7..D5 of n is used to set the printing break time.  Break time
    // is n(D7-D5)*250us.
    // (Unsure of the default value for either -- not documented)
    
#define printDensity   14 // 120% (? can go higher, text is darker but fuzzy)
#define printBreakTime  4 // 500 uS
    
    send(18);
    send(35); // DC2 # (print density)
    
    send((printBreakTime << 5) | printDensity);
    
#define dotPrintTime = 30000; // See comments near top of file for
#define dotFeedTime  =  2100; // an explanation of these values.
#define maxChunkHeight = 255;

}


void feedPrinter(uint8_t x) {
    while(x--) send('\n');
}

#define TIMEOUTLIMIT 10000
uint8_t reccam() {
    uint timeout = 0;
    while ((!(USART_GetFlagStatus(USART2,USART_FLAG_RXNE)))&&(timeout<TIMEOUTLIMIT)){
        timeout++;
        Delay(1);
    }
    if (timeout == TIMEOUTLIMIT)
        return 0;
    return USART_ReceiveData(USART2);
}

uint16_t getBufLengthCam() {
    
    uint16_t response;
    
    //clear out buffer I guess?
    reccam();


    //get buff len
    sendcam(0x56);
    sendcam(0x0);
    sendcam(0x34);
    sendcam(0x01);
    sendcam(0x00);
    reccam();
    reccam();
    reccam();
    reccam();
    response = reccam();
    
    if (response==0x04) {
        reccam();
        reccam();
        response = reccam();
        response = response *256;
        response = response | reccam();
        return response;
        
    }
    else {
        return 0;
    }
}



void TIM2_IRQHandler()
{
    if (TIM_GetITStatus(TIM2, TIM_IT_Update) != RESET) {
        
        TIM_ClearITPendingBit(TIM2, TIM_IT_Update);
        
        
        if (LEDBlinkStatus&1) {
            if (getLED(RED))
                GPIO_ResetBits(GPIOA,RED);
            else
                GPIO_SetBits(GPIOA,RED);
        }
        else if (LEDBlinkStatus&2) {
            if (getLED(GREEN))
                GPIO_ResetBits(GPIOA,GREEN);
            else
                GPIO_SetBits(GPIOA,GREEN);
        }

    }

}

uint8_t battPresent() {
    return GPIO_ReadInputDataBit(GPIOB,GPIO_Pin_4);
}

uint8_t chargerPresent() {
    return GPIO_ReadInputDataBit(GPIOB,GPIO_Pin_5);
}

uint8_t charging() {
    return !(GPIO_ReadInputDataBit(GPIOB,GPIO_Pin_2));
}

void chargeEnable() {
    GPIO_SetBits(GPIOC,GPIO_Pin_2);
}

void chargeDisable() {
    GPIO_ResetBits(GPIOC,GPIO_Pin_2);
}

int checkBaudRate(USART_InitTypeDef USART_InitStruct)
{
    int ret;
	
    USART_Cmd(USART2, DISABLE);
    USART_InitStruct.USART_BaudRate = 38400;
    USART_Init(USART2, &USART_InitStruct);
    USART_Cmd(USART2, ENABLE);	
    Delay(0x7FFF);
    sendcam(0x56);
    sendcam(0x0);
    sendcam(0x11);
    sendcam(0x00);
    ret = reccam();
    if (ret == 0x76)
	    return 38400;
    USART_Cmd(USART2, DISABLE);
    USART_InitStruct.USART_BaudRate = 115200;
    USART_Init(USART2, &USART_InitStruct);
    USART_Cmd(USART2, ENABLE);
    Delay(0x7FFF);
    sendcam(0x56);
    sendcam(0x0);
    sendcam(0x11);
    sendcam(0x00);
    
    ret = reccam();
    if (ret == 0x76)
	    return 115200;
    else return 0;
}

firstfewbytes[10]={0,0,0,0,0,0,0,0,0,0};

int main(void)
{
    
    //Disable STDOUT buffering. Otherwise nothing will be printed before
    //a newline character or when the buffer is flushed. This MUST be done
    //before any writes to STDOUT to have any effect...
    //setbuf(stdout, NULL);
    
  /*!< At this stage the microcontroller clock setting is already configured, 
       this is done through SystemInit() function which is called from startup
       file (startup_stm32f4xx.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
        system_stm32f4xx.c file
     */


    /* GPIOD Periph clock enable */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD, ENABLE);
    /* Configure PD12, PD13, PD14 and PD15 in output pushpull mode */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12 | GPIO_Pin_13| GPIO_Pin_14| GPIO_Pin_15;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOD, &GPIO_InitStructure);
    

    //db_printf("Entered Main\r\n");
    
    /* GPIOC Periph clock enable */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);
    /* configure PC1/PC2/PC3 (camera/charger/Printer enable) in push pull out mode */
    GPIO_SetBits(GPIOC,GPIO_Pin_1); //keep camera off by default
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1|GPIO_Pin_2|GPIO_Pin_3;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOC, &GPIO_InitStructure);

    
    
    //configure LEDs
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2,ENABLE);
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
    
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9|GPIO_Pin_10|GPIO_Pin_15;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    //configure charger interface pins
    /* GPIOB Periph clock enable */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
    /* configure PB1, PB2 as pull up inputs and PB4, and PB6 as regular inputs */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1|GPIO_Pin_2;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4|GPIO_Pin_5;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    
    
    
    TIM_TimeBaseInitTypeDef timerInitStructure;
    timerInitStructure.TIM_Period = 100;
    timerInitStructure.TIM_CounterMode = TIM_CounterMode_Up;
    timerInitStructure.TIM_Prescaler = 65535;
    timerInitStructure.TIM_ClockDivision = TIM_CKD_DIV1;
    timerInitStructure.TIM_RepetitionCounter = 0;
    TIM_TimeBaseInit(TIM2, &timerInitStructure);
    
    TIM_ITConfig(TIM2,TIM_IT_Update,ENABLE);
    TIM_Cmd(TIM2,ENABLE);

    
    NVIC_InitTypeDef nvicStructure;
    nvicStructure.NVIC_IRQChannel = TIM2_IRQn;
    nvicStructure.NVIC_IRQChannelPreemptionPriority = 0;
    nvicStructure.NVIC_IRQChannelSubPriority = 1;
    nvicStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&nvicStructure);
    

    //checkBatteryChargeStatus();
    setCamera(1);
    
    setLED(GREEN,BLINK);

   
    
    
    
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
    GPIO_Init(GPIOA,&GPIO_InitStructure);
    
    
    
    

    /* Configure Printer UART */
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_UART4, ENABLE);
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO,ENABLE);
    //GPIO_PinAFConfig(GPIOC, GPIO_PinSource10,GPIO_AF_UART4);
    //GPIO_PinAFConfig(GPIOC, GPIO_PinSource11,GPIO_AF_UART4);
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOC, &GPIO_InitStructure);
    USART_InitTypeDef USART_InitStruct;
    USART_StructInit(&USART_InitStruct);
    USART_InitStruct.USART_BaudRate = 19200;
    USART_Init(UART4, &USART_InitStruct);
    USART_Cmd(UART4, ENABLE);

    

    

    
    /* configure camera USART */
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO,ENABLE);
    //GPIO_PinAFConfig(GPIOA, GPIO_PinSource2,GPIO_AF_USART2);
    //GPIO_PinAFConfig(GPIOA, GPIO_PinSource3,GPIO_AF_USART2);
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    
    USART_StructInit(&USART_InitStruct);
    USART_InitStruct.USART_BaudRate = 38400;
    USART_Init(USART2, &USART_InitStruct);
    USART_Cmd(USART2, ENABLE);
    
    

    
    // if camera baud is set baud to 38400, set it to 115200
    while (!(checkBaudRate(USART_InitStruct) == 115200))
    {
	USART_Cmd(USART2, DISABLE);
	USART_InitStruct.USART_BaudRate = 38400;
	USART_Init(USART2, &USART_InitStruct);
	USART_Cmd(USART2, ENABLE);	
        sendcam(0x56);
        sendcam(0x0);
        sendcam(0x24);
        sendcam(0x03);
        sendcam(0x01);
        sendcam(0x0D);
        sendcam(0xA6);
        Delay(0x1FFF);
    }
    
    //set UART to 115200 to match camera
    USART_Cmd(USART2, DISABLE);
    USART_InitStruct.USART_BaudRate = 115200;
    USART_Init(USART2, &USART_InitStruct);
    USART_Cmd(USART2, ENABLE);
    
    /*
    //reset
    Delay(0xFFFF);
    sendcam(0x56);
    sendcam(0x00);
    sendcam(0x26);
    sendcam(0x00);
*/

    //startup the printer
    initThermalPrinter();

    Delay(0xFFFF);
    
  
    
    while(1) {
        
        
        setLED(GREEN,ON);
        setLED(RED,OFF);
        while ((GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_11)==1)||(chargerPresent())) {
            if (chargerPresent() && (!battPresent())) {
                chargeEnable();
                if (charging()) {
                    setLED(RED,ON);
                    setLED(GREEN,OFF);
                }
                else {
                    setLED(RED,OFF);
                    setLED(GREEN,ON);
                }
                
            }
            else {
                setLED(RED,OFF);
                setLED(GREEN,ON);
                chargeDisable();
            }
            
            
        }
        setLED(RED,BLINK);
        setLED(GREEN,BLINK);
        
        
        
        feedPrinter(2);
        
        //take photo
        sendcam(0x56);
        sendcam(0x00);
        sendcam(0x36);
        sendcam(0x01);
        sendcam(0x02);
        Delay(0xFFFF);
        sendcam(0x56);
        sendcam(0x0);
        sendcam(0x36);
        sendcam(0x01);
        sendcam(0x00);

        
        //this is a temporary storage space for the decoded bitmap
        //it's stored in a format ready for the printer.
        uint8_t bmpBlock[60][640];
        
        //this keeps track of how many 8x8 blocks have been processed
        //this is important because the main loop counts on checking
        //decode status to figure out when it should stop decoding
        //I need to know when x number of blocks have gone through in order
        //to print them.
        uint32_t colcount;
        
        
        int gammaCounter;
        
            colcount = 0;
        for (gammaCounter=1;gammaCounter<2;gammaCounter++) {
            //reset image counter
            localImageBufferCursor = 0;
            //reset packet counter
            arrayCursor = 0;

            g_nInFileSize = 0;
            readBufferIndex = BUFFER_INDEX_START;
            writeBufferIndex = BUFFER_INDEX_START;
            done = 0;
            //main loop already filled the first part of &picture
            localBufferAddr = &picture[BUFFER_INDEX_START][0];
            altBufferAddr = &picture[BUFFER_INDEX_START][0];
                
            while (g_nInFileSize ==0)  {
                g_nInFileSize = getBufLengthCam();
            }
            
            Delay(0xFFFF);
            
            //grab incoming image data
            localBufferAddr = readBuffer(localImageBufferCursor,LOCAL_IMAGE_BUFFER_SIZE,&picture[0][0]);
            waitForDMAComplete();
            
            uint8_t i;
            for (i=0;i<10;i++) {
                firstfewbytes[i] = picture[0][i];
            }
            
            //donothing();
            
            //resume frame
            /*sendcam(0x56);
            sendcam(0x00);
            sendcam(0x36);
            sendcam(0x01);
            sendcam(0x02);
            Delay(0xFFFFF);*/

            
            
            
            
            //new image object
            pjpeg_image_info_t image_info;
            int mcu_x = 0;
            int mcu_y = 0;
            uint32_t row_pitch;
            uint8_t *pImage;
            uint8_t status;
            
            //the JPEG decoder can only do a few 8x8 blocks at a time
            //these values let you know how many it's done this time around
            uint32_t row_blocks_per_mcu, col_blocks_per_mcu;
            
            
            g_nInFileOfs=0;
            
            //reduce is a fancy function that just picks a representative byte from each 8x8 block
            int reduce=0;

            //startup your decoder
            //db_printf("Decode Init\r\n");
            status = pjpeg_decode_init(&image_info, pjpeg_need_bytes_callback, NULL, (unsigned char)reduce);

            
            
            
            
            
            //figure out how many 8x8 blocks you will be decoding each round
            row_blocks_per_mcu = image_info.m_MCUWidth >> 3;
            col_blocks_per_mcu = image_info.m_MCUHeight >> 3;
            
            

            
            //this stores the remaining dither values on the right side of each column
            int32_t rightDither[8] = {0,0,0,0,0,0,0,0};
            
            //the same thing, but at the bottom of each row
            int32_t downDither[640];
            int l;
            for (l=0;l<640;l++) {
                downDither[l]=0;
            }
            

            
            
            for ( ; ; )
            {
                int y, x;

                status = pjpeg_decode_mcu();

                if (status)
                {
                    //db_printf("Out of blocks?\r\n");
                    if (status != PJPG_NO_MORE_BLOCKS)
                    {
                        //db_printf("pjpeg_decode_mcu() failed\r\n");
                        
                        
                        
                        setLED(RED,ON);
                        setLED(GREEN,OFF);
                        while (GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_11)==1);
                        sendcam(0x56);
                        sendcam(0x0);
                        sendcam(0x32);
                        sendcam(0x0C);
                        sendcam(0x00);
                        sendcam(0x0A);
                        sendcam((0>>24)&0xff);
                        sendcam((0>>16)&0xff);
                        sendcam((0>>8 )&0xff);
                        sendcam((0    )&0xff);
                        sendcam((g_nInFileSize>>24)&0xff);
                        sendcam((g_nInFileSize>>16)&0xff);
                        sendcam((g_nInFileSize>>8)&0xff);
                        sendcam((g_nInFileSize)&0xff);//32 bytes at a time
                        sendcam(0);
                        sendcam(0);
                        while(1);
                        
                    }
                    
                    break;
                }
                
                
                
                for (y = 0; y < image_info.m_MCUHeight; y += 8)
                {
                    const int by_limit = min(8, image_info.m_height - (mcu_y * image_info.m_MCUHeight + y));
                    
                    for (x = 0; x < image_info.m_MCUWidth; x += 8)
                    {
                        
                        // Compute source byte offset of the block in the decoder's MCU buffer.
                        uint src_ofs = (x * 8U) + (y * 16U);
                        const uint8_t *pSrcR = image_info.m_pMCUBufR + src_ofs;
                        const uint8_t *pSrcG = image_info.m_pMCUBufG + src_ofs;
                        const uint8_t *pSrcB = image_info.m_pMCUBufB + src_ofs;
                        
                        const int bx_limit = min(8, image_info.m_width - (mcu_x * image_info.m_MCUWidth + x));
                        
                        
                        int i;
                        int j;

                        //grab raw data from image
                        int32_t ditheredImage[8][8];
                        for (i=0;i<8;i++) {
                            for (j=0;j<8;j++) {
                                ditheredImage[i][j] = (gammaCurve[gammaCounter][*(pSrcR++)])*210+(gammaCurve[gammaCounter][*(pSrcG++)])*720+(gammaCurve[gammaCounter][*(pSrcB++)])*70;
                                
                            }
                        }
                        
                        //add dithering overflow from adjascent block to the left and top
                        int k;
                        for (k=0;k<8;k++) {
                            ditheredImage[0][k] += rightDither[k];
                            rightDither[k]=0;
                            ditheredImage[k][0] += downDither[k+x];
                            downDither[k+x]=0;
                        }

                        
                        //the following performs the actual dithering
                        //it's basically adding up the quantization error
                        //and letting it spill over to the right and bottom

                        for (i=0;i<8;i++) {
                            for (j=0;j<8;j++) {
                                int32_t oldpixel = ditheredImage[i][j];
                                int32_t newpixel = 0;
                                
                                //round pixel up or down
                                if (oldpixel<80000) {
                                    newpixel = 0;
                                }
                                else {
                                    newpixel = 255000;
                                }
                                ditheredImage[i][j]=newpixel;
                                
                                //calculate error
                                int32_t quant_error = oldpixel-newpixel;
                                
                                //add error to pixels to the right
                                if (i<7) {
                                    ditheredImage[i+1][j ] += ((quant_error*7000)/16000);//* (float)(7.0/16);
                                    if (j<7) {
                                        ditheredImage[i+1][j+1] += ((quant_error*1000)/16000);//* (float)(1.0/16);
                                    }
                                }
                                //store dithering info for 8x8 block to the right
                                else {
                                    rightDither[j] += ((quant_error*7000)/16000);//* (float)(7.0/16);
                                    if (j<7) {
                                        rightDither[j+1] += ((quant_error*1000)/16000);//* (float)(1.0/16);
                                    }
                                }
                                
                                //add error to pixels to the bottom
                                if (j<7) {
                                    ditheredImage[i  ][j+1] += ((quant_error*5000)/16000);//*(float)(5.0/16);
                                    if (i>0) {
                                        ditheredImage[i-1][j+1] += ((quant_error*3000)/16000);//* (float)(3.0/16);
                                    }
                                }
                                //store dithering info for 8x8 block to the bottom
                                else {
                                    downDither[x+i] += ((quant_error*5000)/16000);//*(float)(5.0/16);
                                    if (i>0) {
                                        downDither[x+i-1]+= ((quant_error*3000)/16000);//* (float)(3.0/16);
                                    }
                                }
                            }
                        }

                        
                        
                        
                        
                        //take output of dithering algorithm, round up or down
                        //and format it for the printer
                        //the printer takes one byte at a time which represents 8 pixels
                        //(don't ask me the orientation, I just guessed and checked)
                        int bx, by;
                        for (by = 0; by < by_limit; by++)
                        {
                            for (bx = 0; bx < bx_limit; bx++)
                            {
                                //this part is a little goofy.
                                //each row of the image is 80 blocks wide
                                //so colcount%80 tells you how far you are
                                //through the current row of blocks
                                //each block is one printer-byte wide
                                //likewise, colcount/80 is the number of
                                //vertical block rows you've traveled
                                //because printer data goes left-right (I think?)
                                //each byte is only one pixel high.
                                //you need to multiply by 8 to figure out how many
                                //pixels down you are.
                                if (ditheredImage[bx][by]<128000)
                                    bmpBlock[(colcount/80)][639-by-((colcount%80)*8)]|=(1<<(7-bx));
                                else
                                    bmpBlock[(colcount/80)][639-by-((colcount%80)*8)]&=(~(1<<(7-bx)));
                                
                                /*if (ditheredImage[bx][by] < 128)
                                    bmpBlock[colcount%80][bx+(colcount/80)*8]|=(1<<(7-by));
                                else
                                    bmpBlock[colcount%80][bx+(colcount/80)*8]&=(~(1<<(7-by)));
                                 */
                            }
                            
                        }
                        //you did a single 8x8 block of data
                        colcount++;
                        

                    }
                }
                
                
                mcu_x++;
                if (mcu_x == image_info.m_MCUSPerRow)
                {
                    mcu_x = 0;
                    mcu_y++;
                }
                
            }
        }
            
            
            
            
            waitForDMAComplete();
            Delay(100000);
        
        setLED(RED,ON);
        setLED(GREEN,ON);

        
            
            
            RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3,ENABLE);
            //TIM_TimeBaseInitTypeDef timerInitStructure;
            timerInitStructure.TIM_Period = 10;
            timerInitStructure.TIM_CounterMode = TIM_CounterMode_Up;
            timerInitStructure.TIM_Prescaler = 5616;
            timerInitStructure.TIM_ClockDivision = TIM_CKD_DIV1;
            timerInitStructure.TIM_RepetitionCounter = 0;
            TIM_TimeBaseInit(TIM3, &timerInitStructure);
            TIM_ITConfig(TIM3, TIM_IT_Update, ENABLE);
            TIM_Cmd(TIM3,ENABLE);
                
            //there are 80 blocks per row
            //each block is 8 high
            //so after 240 rows, expect
            //(240/8)*80 = 2400 colcounts
            //db_printf("Printing!\r\n");
            send(18);
            send(42);
            send(240);//rows in each chunk
            send(48);//clipped to 384px wide
            int px, py;
            for (py=0;py<240;py++) {
                for (px=6;px<54;px++) {
                    TIM_ClearITPendingBit(TIM3, TIM_IT_Update);
                    while (TIM_GetITStatus(TIM3, TIM_IT_Update) == RESET)
                    {}
                    send(bmpBlock[px][py]);
                    }
            }
            send(18);
            send(42);
            send(240);//rows in each chunk
            send(48);//clipped to 384px wide
            for (py=240;py<480;py++) {
                for (px=6;px<54;px++) {
                    TIM_ClearITPendingBit(TIM3, TIM_IT_Update);
                    while (TIM_GetITStatus(TIM3, TIM_IT_Update) == RESET)
                    {}
                    send(bmpBlock[px][py]);
                }
            }
            send(18);
            send(42);
            send(160);//rows in each chunk
            send(48);//clipped to 384px wide
            for (py=480;py<640;py++) {
                for (px=6;px<54;px++) {
                    TIM_ClearITPendingBit(TIM3, TIM_IT_Update);
                    while (TIM_GetITStatus(TIM3, TIM_IT_Update) == RESET)
                    {}
                    send(bmpBlock[px][py]);
                }
            }
            feedPrinter(4);
        
    }
}

/**
  * @brief  Delay Function.
  * @param  nCount:specifies the Delay time length.
  * @retval None
  */
void Delay(__IO uint32_t nCount)
{
    uint32_t temp = nCount;
  while(nCount--)
  {
  }
    nCount = temp;
    while(nCount--)
    {
    }
    nCount = temp;
    while(nCount--)
    {
    }
}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

/**
  * @}
  */ 

/**
  * @}
  */ 

/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
